# Generated by Django 2.1 on 2018-08-26 01:14

import django.contrib.gis.db.models.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AllFieldsModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('boolean', models.BooleanField()),
                ('char', models.CharField(max_length=32)),
                ('file', models.FileField(upload_to='')),
                ('image', models.ImageField(upload_to='')),
                ('slug', models.SlugField()),
                ('text', models.TextField()),
                ('date', models.DateField()),
                ('datetime', models.DateTimeField()),
                ('float', models.FloatField()),
                ('biginteger', models.BigIntegerField()),
                ('decimal', models.DecimalField(decimal_places=0, max_digits=9)),
                ('integer', models.IntegerField()),
                ('positiveinteger', models.PositiveIntegerField()),
                ('positivesmallinteger', models.PositiveSmallIntegerField()),
                ('smallinteger', models.SmallIntegerField()),
                ('time', models.TimeField()),
                ('geometry', django.contrib.gis.db.models.fields.GeometryField(srid=4326)),
                ('point', django.contrib.gis.db.models.fields.PointField(srid=4326)),
                ('linestring', django.contrib.gis.db.models.fields.LineStringField(srid=4326)),
                ('polygon', django.contrib.gis.db.models.fields.PolygonField(srid=4326)),
                ('multipoint', django.contrib.gis.db.models.fields.MultiPointField(srid=4326)),
                ('multilinestring', django.contrib.gis.db.models.fields.MultiLineStringField(srid=4326)),
                ('multipolygon', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326)),
                ('geometrycollection', django.contrib.gis.db.models.fields.GeometryCollectionField(srid=4326)),
            ],
        ),
        migrations.CreateModel(
            name='ModelA',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, unique=True)),
                ('category', models.CharField(max_length=64)),
                ('blank', models.CharField(blank=True, max_length=64)),
            ],
        ),
        migrations.CreateModel(
            name='ModelB',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, unique=True)),
                ('nullable_a1', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='testapp.ModelA')),
                ('nullable_a2', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='testapp.ModelA')),
                ('required_a', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='testapp.ModelA')),
            ],
        ),
    ]
